from setuptools import setup

setup(
    name='RP-DataBase',
    version='1.0.1',
    packages=['RPDB'],
    url='https://github.com/hsn8086/RPDB',
    license='MIT license',
    author='hsn8086',
    author_email='hsn1919810@gmail.com',
    description='a database'
)
